==> multiqc-0.7.1dev0-py27_0.tar.bz2: lib/python2.7/site-packages/multiqc-0.8.dev0-py2.7.egg/multiqc/modules/samtools/flagstat.py <==
# coding: utf-8
#!/usr/bin/env python

""" MultiQC submodule to parse output from Samtools flagstat """

import logging
import re
from collections import OrderedDict, defaultdict
from multiqc import config, plots 

# Initialise the logger
log = logging.getLogger(__name__)

class FlagstatReportMixin():

    def parse_samtools_flagstats(self):
        """ Find Samtools flagstat logs and parse their data """

        self.samtools_flagstat = dict()
        for f in self.find_log_files(config.sp['samtools']['flagstat']):
            parsed_data = parse_single_report(f['f'])
            if len(parsed_data) > 0:
                if f['s_name'] in self.samtools_flagstat:
                    log.debug("Duplicate sample name found! Overwriting: {}".format(f['s_name']))
                self.add_data_source(f)
                self.samtools_flagstat[f['s_name']] = parsed_data

        if len(self.samtools_flagstat) > 0:

            # Write parsed report data to a file (restructure first)
            self.write_data_file(self.samtools_flagstat, 'multiqc_samtools_flagstat')
            
            # General Stats Table
            flagstats_headers = dict()
            flagstats_headers['mapped_passed'] = {
                'title': 'M Reads Mapped',
                'description': 'Reads Mapped in the bam file',
                'min': 0,
                'modify': lambda x: x / 1000000,
                'shared_key': 'read_count'
            }
            self.general_stats_addcols(self.samtools_flagstat, flagstats_headers, 'Samtools Flagstat')
            
            # Make dot plot of counts
            keys = OrderedDict()
            reads = {
                'min': 0,
                'modify': lambda x: float(x) / 1000000.0,
                'suffix': 'M reads',
                'decimalPlaces': 2,
                'shared_key': 'read_count'
            }
            keys['flagstat_total'] = dict(reads, **{'title': 'Total Reads' })
            keys['total_passed'] = dict(reads, **{'title': 'Total Passed QC' })
            keys['mapped_passed'] = dict(reads, **{'title': 'Mapped' })
            keys['secondary_passed'] = dict(reads, **{'title': 'Secondary Alignments' })
            keys['supplementary_passed'] = dict(reads, **{'title': 'Supplementary Alignments' })
            keys['duplicates_passed'] = dict(reads, **{'title': 'Duplicates' })
            keys['paired in sequencing_passed'] = dict(reads, **{'title': 'Paired in Sequencing' })
            keys['properly paired_passed'] = dict(reads, **{'title': 'Properly Paired' })
            keys['with itself and mate mapped_passed'] = dict(reads, **{'title': 'Self and mate mapped', 'description': 'Reads with itself and mate mapped' })
            keys['singletons_passed'] = dict(reads, **{'title': 'Singletons' })
            keys['with mate mapped to a different chr_passed'] = dict(reads, **{'title': 'Mate mapped to diff chr', 'description': 'Mate mapped to different chromosome' })
            keys['with mate mapped to a different chr (mapQ >= 5)_passed'] = dict(reads, **{'title': 'Diff chr (mapQ >= 5)', 'description':'Mate mapped to different chromosome (mapQ >= 5)' })
            
            self.sections.append({
                'name': 'Samtools Flagstat',
                'anchor': 'samtools-flagstat',
                'content': '<p>This module parses the output from <code>samtools flagstat</code>. All numbers in millions.</p>' +
                            plots.beeswarm.plot(self.samtools_flagstat, keys, {'id': 'samtools-flagstat-dp'})
            })
        
        # Return the number of logs that were found
        return len(self.samtools_flagstat)


# flagstat has one thing per line, documented here (search for flagstat):
# http://www.htslib.org/doc/samtools.html 
flagstat_regexes = {
    'total':         r"(\d+) \+ (\d+) in total \(QC-passed reads \+ QC-failed reads\)", 
    'secondary':     r"(\d+) \+ (\d+) secondary",
    'supplementary': r"(\d+) \+ (\d+) supplementary", 
    'duplicates':    r"(\d+) \+ (\d+) duplicates", 
    'mapped':        r"(\d+) \+ (\d+) mapped \(([\d\.]+|-?nan)%:([\d\.]+|-?nan)%\)", 
    'paired in sequencing': r"(\d+) \+ (\d+) paired in sequencing", 
    'read1':         r"(\d+) \+ (\d+) read1", 
    'read2':         r"(\d+) \+ (\d+) read2", 
    'properly paired': r"(\d+) \+ (\d+) properly paired \(([\d\.]+|-?nan)%:([\d\.]+|-?nan)%\)", 
    'with itself and mate mapped': r"(\d+) \+ (\d+) with itself and mate mapped", 
    'singletons':    r"(\d+) \+ (\d+) singletons \(([\d\.]+|-?nan)%:([\d\.]+|-?nan)%\)",
    'with mate mapped to a different chr': r"(\d+) \+ (\d+) with mate mapped to a different chr",
    'with mate mapped to a different chr (mapQ >= 5)': r"(\d+) \+ (\d+) with mate mapped to a different chr \(mapQ>=5\)",
}

def parse_single_report(file_obj):
    """
    Take a filename, parse the data assuming it's a flagstat file
    Returns a dictionary {'lineName_pass' : value, 'lineName_fail' : value}
    """
    parsed_data = {}

    re_groups = ['passed', 'failed', 'passed_pct', 'failed_pct']
    for k, r in flagstat_regexes.items():
        r_search = re.search(r, file_obj, re.MULTILINE)
        if r_search:
            for i,j in enumerate(re_groups):
                try:
                    key = "{}_{}".format(k, j)
                    parsed_data[key] = float(r_search.group(i+1))
                except IndexError:
                    pass # Not all regexes have percentages
    # Work out the total read count
    try:
        parsed_data['flagstat_total'] = parsed_data['total_passed'] + parsed_data['total_failed']
    except KeyError:
        pass
    return parsed_data

==> multiqc-0.8dev0-py34_0.tar.bz2: lib/python3.4/site-packages/multiqc-0.8.dev0-py3.4.egg/multiqc/modules/samtools/flagstat.py <==
# coding: utf-8
#!/usr/bin/env python

""" MultiQC submodule to parse output from Samtools flagstat """

import logging
import re
from collections import OrderedDict, defaultdict
from multiqc import config, plots

# Initialise the logger
log = logging.getLogger(__name__)

class FlagstatReportMixin():

    def parse_samtools_flagstats(self):
        """ Find Samtools flagstat logs and parse their data """

        self.samtools_flagstat = dict()
        for f in self.find_log_files(config.sp['samtools']['flagstat']):
            parsed_data = parse_single_report(f['f'])
            if len(parsed_data) > 0:
                if f['s_name'] in self.samtools_flagstat:
                    log.debug("Duplicate sample name found! Overwriting: {}".format(f['s_name']))
                self.add_data_source(f, section='flagstat')
                self.samtools_flagstat[f['s_name']] = parsed_data

        if len(self.samtools_flagstat) > 0:

            # Write parsed report data to a file (restructure first)
            self.write_data_file(self.samtools_flagstat, 'multiqc_samtools_flagstat')
            
            # General Stats Table
            flagstats_headers = dict()
            flagstats_headers['mapped_passed'] = {
                'title': 'M Reads Mapped',
                'description': 'Reads Mapped in the bam file',
                'min': 0,
                'modify': lambda x: x / 1000000,
                'shared_key': 'read_count'
            }
            self.general_stats_addcols(self.samtools_flagstat, flagstats_headers, 'Samtools Flagstat')
            
            # Make dot plot of counts
            keys = OrderedDict()
            reads = {
                'min': 0,
                'modify': lambda x: float(x) / 1000000.0,
                'suffix': 'M reads',
                'decimalPlaces': 2,
                'shared_key': 'read_count'
            }
            keys['flagstat_total'] = dict(reads, **{'title': 'Total Reads' })
            keys['total_passed'] = dict(reads, **{'title': 'Total Passed QC' })
            keys['mapped_passed'] = dict(reads, **{'title': 'Mapped' })
            keys['secondary_passed'] = dict(reads, **{'title': 'Secondary Alignments' })
            keys['supplementary_passed'] = dict(reads, **{'title': 'Supplementary Alignments' })
            keys['duplicates_passed'] = dict(reads, **{'title': 'Duplicates' })
            keys['paired in sequencing_passed'] = dict(reads, **{'title': 'Paired in Sequencing' })
            keys['properly paired_passed'] = dict(reads, **{'title': 'Properly Paired' })
            keys['with itself and mate mapped_passed'] = dict(reads, **{'title': 'Self and mate mapped', 'description': 'Reads with itself and mate mapped' })
            keys['singletons_passed'] = dict(reads, **{'title': 'Singletons' })
            keys['with mate mapped to a different chr_passed'] = dict(reads, **{'title': 'Mate mapped to diff chr', 'description': 'Mate mapped to different chromosome' })
            keys['with mate mapped to a different chr (mapQ >= 5)_passed'] = dict(reads, **{'title': 'Diff chr (mapQ >= 5)', 'description':'Mate mapped to different chromosome (mapQ >= 5)' })
            
            self.sections.append({
                'name': 'Samtools Flagstat',
                'anchor': 'samtools-flagstat',
                'content': '<p>This module parses the output from <code>samtools flagstat</code>. All numbers in millions.</p>' +
                            plots.beeswarm.plot(self.samtools_flagstat, keys, {'id': 'samtools-flagstat-dp'})
            })
        
        # Return the number of logs that were found
        return len(self.samtools_flagstat)


# flagstat has one thing per line, documented here (search for flagstat):
# http://www.htslib.org/doc/samtools.html 
flagstat_regexes = {
    'total':         r"(\d+) \+ (\d+) in total \(QC-passed reads \+ QC-failed reads\)", 
    'secondary':     r"(\d+) \+ (\d+) secondary",
    'supplementary': r"(\d+) \+ (\d+) supplementary", 
    'duplicates':    r"(\d+) \+ (\d+) duplicates", 
    'mapped':        r"(\d+) \+ (\d+) mapped \(([\d\.]+|-?nan)%:([\d\.]+|-?nan)%\)", 
    'paired in sequencing': r"(\d+) \+ (\d+) paired in sequencing", 
    'read1':         r"(\d+) \+ (\d+) read1", 
    'read2':         r"(\d+) \+ (\d+) read2", 
    'properly paired': r"(\d+) \+ (\d+) properly paired \(([\d\.]+|-?nan)%:([\d\.]+|-?nan)%\)", 
    'with itself and mate mapped': r"(\d+) \+ (\d+) with itself and mate mapped", 
    'singletons':    r"(\d+) \+ (\d+) singletons \(([\d\.]+|-?nan)%:([\d\.]+|-?nan)%\)",
    'with mate mapped to a different chr': r"(\d+) \+ (\d+) with mate mapped to a different chr",
    'with mate mapped to a different chr (mapQ >= 5)': r"(\d+) \+ (\d+) with mate mapped to a different chr \(mapQ>=5\)",
}

def parse_single_report(file_obj):
    """
    Take a filename, parse the data assuming it's a flagstat file
    Returns a dictionary {'lineName_pass' : value, 'lineName_fail' : value}
    """
    parsed_data = {}

    re_groups = ['passed', 'failed', 'passed_pct', 'failed_pct']
    for k, r in flagstat_regexes.items():
        r_search = re.search(r, file_obj, re.MULTILINE)
        if r_search:
            for i,j in enumerate(re_groups):
                try:
                    key = "{}_{}".format(k, j)
                    parsed_data[key] = float(r_search.group(i+1))
                except IndexError:
                    pass # Not all regexes have percentages
    # Work out the total read count
    try:
        parsed_data['flagstat_total'] = parsed_data['total_passed'] + parsed_data['total_failed']
    except KeyError:
        pass
    return parsed_data
